# -*- coding: utf_8 -*-
# This first line to set encoding is ABSOLUTEY REQUIRED when working with non-latin text
#Check that the NLTK support is adequate to do this

import nltk
import re
import sequential

#This may not be nececessary, but it is a good safety measure
re.UNICODE 


#Pull in A piece of persian text from a file
#handle=codecs.open('PersianTest.txt',encoding='utf-8')
#raw=handle.read().encode("utf-8")
#PersianText=nltk.Text(nltk.word_tokenize(raw))

#Apparently this other tokenizer is sometimes better at handling utf-8, switch if there is a problem
#PersianText=nltk.Text(nltk.PunktWordTokenizer().tokenize(raw))


#To input a string literal, do the following (Broken out in to tiny steps for testing):
# The string here is a random line from the wikipedia article on the language
#PersianTextIn=u'ناحیه پارس و دیگر نقاط ایران بازگشت'

#PersianTextIn=u" نی اگرچه همراه شاید این‌ها گویا را من  این و هشت روشنتر  صادق  حتی حتی آسمانتن  تن گذاشتند"
PersianTextIn=u"شیوه عطسه کردن ، اطلاعات زیادی را در مورد شخصیت انسان می‌دهد. این نکته از یافته های یک پژوهش تازه است. به گزارش سرویس پژوهشی خبرگزاری دانشجویان ایران، به گفته دکتر آلان هیرچ، متخصص مغز و اعصاب بنیاد پژوهش و درمان بویایی و چشایی، یک عطسه بلند و انفجاری احتمالاً از سوی یک شخصیت برونگرا و اجتماعی رخ می‌دهد، در حالیکه یک فرد خجالتی بیشتر تلاش می‌کند تا عطسه آرامتری داشته باشد."
PersianTextIn=PersianTextIn.encode("utf-8")
# Punkt tokenizer breaks in inappropriate places, despite the documentation saying it is better behaved on utf-8
#PersianTokens=nltk.PunktWordTokenizer().tokenize(PersianTextIn)
PersianTokens=nltk.word_tokenize(PersianTextIn)
PersianText=nltk.Text(PersianTokens)


# This is a test to demonstrate that python regexes _do_ work on unicode strings
# It passes as long as we do the "Decode the string before comparision" idiom in the findall line.
#for s in PersianTokens:
#	print s
#	print re.findall(ur'.*و.*',s.decode("utf-8"))



#Some of the Regexes are assembled from prefix/stem classes stored as variables. 
V_ID=u'م|ی|د|یم|ید|ند'
Past_Stem=u'کرد|شد|بود|داشت|گفت|زد|توانست|دانست|گرفت|فهمید|رفت|دید|ساخت|داد|خواست|آمد|گذاشت|یافت|پرسید|آورد|چرخید|شنید|خندید|بوسید|رقصید|خواند|شکفت|خوابید|گشت|افتاد|آموخت|خورد|کوشید|نمود|دزدید|جست|شکست|پذیرفت'
Pres_Stem=u'کن|شو|باش|هست|دار|گوی|زن|توان|دان|گیر|فهم|رو|بین|ساز|ده|خواه|آی|گذار|یاب|پرس|آور|چرخ|شنو|خند|بوس|رقص|خوان|شکف|گرد|افت|آموز|خور|کوش|نمای|دزد|جوی|شکن|پذیر'
Clitic=u'م|ت|ش|مان|تان|شان'


VVDEx="(" + V_ID + ")" + "(" + Past_Stem + ")"


#The following two lines build a regexp tagger from a series of regexes, default last
	# Many of these are actually just using regexes to implment a lexicon tagger
patterns=[
	(ur'^و$|^اما$|^ولی$|^لیکن$|^ولیکن$|^بنابراین$|^یا$|^چون$|^زیرا$|^چنانچه$|^اگرچه$',"CC"),
	(ur'^علیرغم$|^گویا$|^گویی$|^تا$|^وقتی$',"SC"),
	(ur'^در$|^از$|^به$|^با$|^زیر$|^روی$|^همراه$|^تا$|^برای$|^درون$|^داخل$|^علیه$|^بر$|^بالای$|^درباره$|^دربارۀ$|^حدود$|^پشت$|^زیر$|^کنار$|^وسط$|^توسط$|^بوسیله$|^بوسیلۀ$|^مانند$|^همچون$|^مثل$|^برعکس$|^بدون$',"IN"),
	(ur'^این$|^آن$|^اینها$|^آنها$|^این‌ها$|^آن‌ها$|^هیچ$|^هر$|^همه$|^همین$|^همان$|^کدام$',"DT"),
	(ur'^را$',"OM"),
	(ur'^یک$|^دو$|^سه$|^چهار$|^پنج$|^شش$|^هفت$|^هشت$|^ده$|^یازده$|^دوازده$|^سیزده$|^پانزده$|^شانزده$|^هفده$|^هجده$|^نوزده$|^بیست$|^سی$|^چهل$|^پنجاه$|^شصت$|^هفتاد$|^هشتاد$|^نود$|^صد$|^دویست$|^سیصد$|^چهاصد$|^پانصد$|^ششصد$|^هفتصد$|^هشتصد$|^نهصد$|^هزار$|^میلیون$|^میلیارد$|^تریلیارد$',"CD"),
	(ur'^شاید$|^باید$|^بایستی$|^مبادا$|^بلکه$',"MD"),
	(ur'^من$|^تو$|^او$|^وی$|^ما$|^شما$|^آنها$|^ایشان$',"PRP"),
	(ur'^فقط$|^تنها$|^اکنون$|^حالا$|^هم‌اکنون$|^سپس$|^همچنین$|^بسیار$|^خیلی$|^هنوز$|^آنجا$|^اینجا$|^حتی$|^هم$|^دوباره$|^امروز$|^دیروز$|^همیشه$|^هرگز$|^اغلب$|^چندان$',"RB"),
	(ur'^خوب$|^بد$|^کم$|^زیاد$|^بزرگ$|^کوچک$|^فقیر$|^غنی$|^تاریک$|^روشن$|^سایر$|^دیگر$|^جدید$|^مدرن$|^قدیمی$|^کهنه$|^باستانی$|^کهن$|^عالی$|^بهتر$|^بهترین$|^فوق‌العاده$|^عریض$|^طویل$|^دراز$|^بلند$|^کوتاه$|^وسیع$|^مهربان$|^آسان$|^سخت$|^دشوار$|^سهل$|^پیر$|^جوان$|^شاد$|^شاداب$|^خوش$|^سرخوش$|^دارا$|^سرمست$|^مست$|^برنا$|^محکم$|^مستحکم$|^استوار$|^قوی$|^ضعیف$|^موفق$|^محترم$|^محتمل$|^مغلوب$|^حاضر$|^غایب$|^ملی$|^متفاوت$|^شبیه$|^مهم$|^پراهمیت$|^سیاسی$|^اجتماعی$|^اقتصادی$|^ورزشی$|^هنری$|^طولانی$|^راست$|^چپ$|^واقعی$|^خیالی$|^انتزاعی$|^عمومی$|^خصوصی$|^عام$|^خاص$|^مطمئن$|^بردبار$|^صبور$|^انسانی$|^اساسی$|^شرعی$|^قانونی$|^دینی$|^مذهبی$|^مدنی$|^تنها$|^محلی$|^سپاهی$|^ارتشی$|^نظامی$|^کامل$|^ناقص$|^آزاد$|^رها$|^بین‌المللی$|^حقیقی$|^ویژه$|^باز$|^بسته$|^مشخص$|^معین$|^واضح$|^تمیز$|^پاک$|^مطهر$|^نیکو$|^نیک$|^مجرد$|^مطلق$|^طبی$|^پزشکی$|^علمی$|^دیرین$|^اشتباه$|^غلط$|^درست$|^نادرست$|^خارجی$|^رایج$|^معمول$|^مرده$|^زنده$|^طبیعی$|^مصنوعی$|^ساختگی$|^مشابه$|^جسمی$|^روحی$|^روانی$|^مرکزی$|^اصلی$|^فرعی$|^جانبی$|^آماده$|^داغ$|^گرم$|^سرد$|^جدی$|^خشن$|^خشونت‌بار$|^خشونت‌آمیز$|^جدا$|^منفک$|^محیطی$|^مالی$|^خوشحال$|^امن$|^زشت$|^زیبا$|^مناسب$|^محترم$|^جاری$|^مقاوم$|^نهایی$|^پایانی$|^ابتدائی$|^ابتدایی$|^نوین$|^سیاه$|^سفید$|^آبی$|^قرمز$|^زرد$|^بنفش$|^نارنجی$|^سبز$|^صورتی$|^خاکستری$|^نیلی$|^یشمی$|^فیروزه‌ای$|^سرخ$|^طلایی$|^نفره‌ای$',"JJ"),
	(ur'$مند+.|^نا+.|.+بی^',"JJ"),
	(ur'^علی$|^رضا$|^محمد$|^احمد$|^محمود$|^حسین$|^حسن$|^صادق$|^سیاوش$|^سیامک$|^مهران$|^صمد$|^مهدی$|^یوسف$|^یونس$|^یعقوب$|^امیر$|^سجاد$|^باقر$|^موسی$|^عیسی$|^کاظم$|^جعفر$|^قلی$|^تقی$|^نقی$|^جواد$|^حامد$|^حمید$|^سعید$|^مسعود$|^ایرج$|^مرتضی$|^اسد$|^صابر$|^شاهین$|^محسن$|^سامان$|^شایان$|^بهروز$|^بهزاد$|^آرش$|^پویا$|^پوریا$|^سپهر$|^حجت$|^کوروش$|^داریوش$|^سیروس$|^سینا$|^کریم$|^رحیم$|^رحمان$|^هومن$|^منصور$|^کیارش$|^کیانوش$|^خسرو$|^سروش$|^قاسم$|^فریدون$|^فردین$|^جلال$|^جمال$|^جلیل$|^نیما$|^آرمین$|^آرمان$|^عارف$|^نادر$|^ناصر$|^منوچهر$|^ایران$|^امریکا$|^انگلیس$|^انگلستان$|^هند$|^ژاپن$|^چین$|^اروپا$|^آسیا$|^فرانسه$|^ایتالیا$|^اسپانیا$|^آلمان$|^هلند$|^استرالیا$|^مغولستان$|^روسیه$|^آذربایجان$|^ارمنستان$|^تهران$|^اصفهان$|^مشهد$|^شیراز$|^اهواز$|^تبریز$|^کرمان$|^یزد$|^بوشهر$|^رشت$|^اردبیل$|^همدان$|^کردستان$|^سنندج$|^قزوین$|^کرج$|^خراسان$|^خوزستان$|^فارس$|^ایلام$|^لرستان$|^زاهدان$|^سیستان$|^بلوچستان$|^پاکستان$|^افغانستان$|^آفریقا$|^افریقا$|^آمریکا$|^اراک$|^کاشان$|^مازندران$|^گیلان$|^فروردین$|^اردیبهشت$|^خرداد$|^تیر$|^مرداد$|^شهریور$|^مهر$|^آبان$|^آذر$|^دی$|^بهمن$|^اسفند$|^جمعه$',"NNP"),
	(ur'$شنبه+.',"NNP"),
	(ur'^آسمان$|^باران$|^امان$|^وجدان$|^جهان$|^ایمان$|^ختم به بان$|^ختم به دان$|^اطمینان$|^زمان$',"NN"),
	(ur'.+دان$|.+بان$|.+گاه$|.+ش$|.+ت$',"NN"),
	(ur'^چه$|^کدام$|^کجا$',"WDT"),
	(ur'.+دن$|.+تن$',"GND"),
	(ur'.+ها$|$های.+|$ان.+|$ات.+|.+اتی$|.+انی$',"NNS"),
	(ur'.+تر$',"JJR"),
	(ur'.+ترین$',"JJS"),
	(ur'$خواهند^|$خواهد^',"AUX"),
	(ur'^\.$',"DOT"),
	(VVDEx,"VVD"),
	(ur'.*',"NN")]
#regexp_tagger=(nltk.RegexpTagger(patterns)) -- This would invoke the normal nltk RegexpTagger
# Use our own mutilated RegexpTagger that isn't broken on unicode
regexp_tagger=(sequential.RegexpTagger(patterns))

#Rolling our own simple digram tagger second pass, rather than involving nltk's NgramTagger


PT=''#Initialize PT to the empty string so the loop is correct on the first word
wprev=('','')
#Currently structured such that the regex tagger _tries_ for each token, but we can override it
for w in regexp_tagger.tag(PersianText):
	#Test the previous tag to set the next tag
	#Nouns follow Prepositions 
	if PT == "PP": #If the previous token had this token
		w[1] == "NN" #This becomes the tag for the current token
	if w[1] == "DOT":
		wprev = (wprev[0],"V")
		print nltk.tag.tuple2str(wprev)
	print nltk.tag.tuple2str(wprev)
	PT = w[1] #Trap the previous tag
	wprev = w; #Store the previous word
print nltk.tag.tuple2str(w)

#To print on one line (Hard to look at)
#tagout= regexp_tagger.tag(PersianText)
#output = ' '.join([nltk.tag.tuple2str(w) for w in tagout])
#print output